--[[----------------------------------------------------------------------------

  Author : Alexander Stoica
  Creation Date : 07/26/2010
  Last modified : 10/04/2010
  Version : 1.01

----------------------------------------------------------------------------]]--

_AUTO_RELOAD_DEBUG = true

--[[ locals ]]--------------------------------------------------------------]]--

local menu_name = "Mixer:Track:"
local keybind_name = "Mixer:Set Track Delay In Samples:"

local sample_rates = {
  "22.050",
  "44.100",
  "48.000",
  "88.200",
  "96.000"
}

--[[ preferences ]]---------------------------------------------------------]]--

prefs = renoise.Document.create("ScriptingToolPreferences") {
  prf_sample_rate = 2
}

renoise.tool().preferences = prefs

--[[ menus, keybindings and notifiers ]]------------------------------------]]--

renoise.tool():add_menu_entry {
  name = menu_name .. "Set Track Delay In Samples",
  invoke = function() set_delay_gui() end,
  active = function() return menu_active() end
}

renoise.tool():add_keybinding {
  name = keybind_name .. "Set Track Delay In Samples",
  invoke = function(repeated) set_delay_gui() end
}

--[[ enable / disable menu entry ]]-----------------------------------------]]--

function menu_active()

  if renoise.song().selected_track.type ==
     renoise.Track.TRACK_TYPE_SEQUENCER then

    return true
  end

  return false

end

--[[ set delay dialog ]]----------------------------------------------------]]--

function set_delay_gui()

  if renoise.song().selected_track.type ~=
     renoise.Track.TRACK_TYPE_SEQUENCER then

    return
  end

  local sample_rate = sample_rates[prefs.prf_sample_rate.value]
  local current_delay = renoise.song().selected_track.output_delay

  local vb = renoise.ViewBuilder()
  local DIALOG_MARGIN = renoise.ViewBuilder.DEFAULT_DIALOG_MARGIN
  local CONTROL_MARGIN = renoise.ViewBuilder.DEFAULT_CONTROL_MARGIN

  local row_sample_rate = vb:vertical_aligner {
    margin = CONTROL_MARGIN,
    spacing = 4,

    vb:text {
      text = "Sample Rate:"
    },
    vb:popup {
      id = "sample_rates_list",
      width = 100,
      items = sample_rates,
      value = prefs.prf_sample_rate.value,
      notifier = function(index)
        set_sample_rates_list(vb.views, index)
        sample_rate = sample_rates[index]
      end
    }
  }

  local row_delay = vb:vertical_aligner {
    margin = CONTROL_MARGIN,
    spacing = 4,

    vb:text {
      text = "Delay In Samples:"
    },
    vb:valuebox {
      id = "track_delay",
      width = 100,
      min = -100 / (1 / sample_rate),
      max = 100 / (1 / sample_rate),
      value = current_delay / (1 / sample_rate)
    }
  }

  local dialog_content = vb:column {
    margin = DIALOG_MARGIN,

    vb:column {
      margin = DIALOG_MARGIN,
      style = "group",

      row_sample_rate,
      row_delay
    }
  }

  local dialog_value = renoise.app():show_custom_prompt (
    "Set Track Delay", dialog_content, {"OK"}
  )

  if dialog_value == "OK" then

    local delay = vb.views.track_delay.value * (1 / sample_rate)

    renoise.song().selected_track.output_delay = delay
    prefs.prf_sample_rate.value = vb.views.sample_rates_list.value

    renoise.app():show_status (
      "Track delay time was set to: " .. delay .. " ms"
    )
  end

end

--[[ sample_rates_list popup handling ]]------------------------------------]]--

function set_sample_rates_list(vbv, i)

  local min = -100 / (1 / sample_rates[i])
  local max = 100 / (1 / sample_rates[i])

  if vbv.track_delay.value < min then
    vbv.track_delay.value = min
  elseif vbv.track_delay.value > max then
    vbv.track_delay.value = max
  end

  vbv.track_delay.min = min
  vbv.track_delay.max = max

end

---------------------------------------------------------------------[[ EOF ]]--
